/**************/
/* laskinr.c  */
/****************************************************************************/
/*
** Windows: Win 3.1 & Win32
**
** Alkeellinen taskulaskin.
**
** Ohjelman kntmiseksi tarvitaan projekti, jossa on
**   laskinr.c
**   laskurir.def
**   laskukor.rc
**
** Vesa Lappalainen
** 18.9.1992, 18.8.1994 (32-bit yteensopivuus)
**
** Tehtvi: 1)  Laita jakolasku toimimaan.
**
**           2)  Tee etumerkin vaihtonppin ja tarvittavat toiminnot.
**
**           3)  Lis muita laskintoimintoja
**
**           4)  Kirjoita ohjelmasta HP-laskinten tyylinen pinolaskin
**               jossa nytss nkyy 4 tulosta ja nytt
**               kyttytyvt seuraavasti:
**
**                nppily:      nytt: X,      Y ,       Z ,      T
**                 [clear all]           0       0         0        0
**                 1                     1       0         0        0
**                 23                  123       0         0        0
**                 [Enter]             123     123         0        0
**                 1                     1     123         0        0
**                 0                    10     123         0        0
**                 [+]                 133       0         0        0
**                 4                     4     133         0        0
**
**               Eli [Enter] pyrytt koko pinoa pykln oikealle
**               monistaen X rekisterin Y rekisteriin.
**               ja operaatiot laskevat aina X ja Y rekisterin vlill
**               pudottaen samalla pinoa se: X = XopY, Y = Z, Z = T
**               Operaation jlkeen ensimminen numeron painallus
**               kopioi T = Z, Z = Y, Y = X,  ja rupeaa tyttmn uutta
**               numero X:n...
**
**
**
*/
/****************************************************************************/
#include <windowsx.h>           /* Tarvitaan 32-bit yhteensopivuudeksi      */
#include "laskinr.h"

#define LASKUREITA 2
#define EKA_LASKURI TULOS

typedef enum {              /* Tunnetut operaattorit:                       */
  EI_OPER,
  PLUS,
  MIINUS,
  KERTO,
  JAKO,
  LASKE
} operaatio_tyyppi;

static HWND GhWnd;

static long laskurit[LASKUREITA] = {0,0};
#define TULOSL     0
#define VALITULOSL 1

/***************************************************************************/
int ShowStaticL(HWND hWnd, int id, long value)
/* Aliohjelmalla tulostetaan Static ikkunaan numero                        */
{
  char s[20];
  wsprintf(s,"%ld ",value);
  SetDlgItemText(hWnd,id,s);
  return 0;
}


/****************************************************************************/
void nayta_laskuri(int laskuri)
{
  ShowStaticL(GhWnd,laskuri+EKA_LASKURI,laskurit[laskuri]);
}

/****************************************************************************/
void lisaa_numero(int laskuri, int nro)
{
  laskurit[laskuri] *= 10;
  laskurit[laskuri] += nro;
  nayta_laskuri(laskuri);
}


/****************************************************************************/
void lisaa_laskuria(int laskuri)
{
  laskurit[laskuri]++;
  nayta_laskuri(laskuri);
}


/****************************************************************************/
void bs_numero(int laskuri)
{
  laskurit[laskuri] /= 10;
  nayta_laskuri(laskuri);
}


static operaatio_tyyppi edel_oper = EI_OPER;
/****************************************************************************/
void oper(
  int valitulos,
  int tulos,
  operaatio_tyyppi operaatio)
{

  switch (edel_oper) {
    case LASKE:   if ( laskurit[valitulos] == 0 ) break;
    case EI_OPER: laskurit[tulos] =  laskurit[valitulos]; break;
    case PLUS:    laskurit[tulos] += laskurit[valitulos]; break;
    case KERTO:   laskurit[tulos] *= laskurit[valitulos]; break;
    case MIINUS:  laskurit[tulos] -= laskurit[valitulos]; break;
  }

  edel_oper = operaatio;
  laskurit[valitulos] = 0;
  nayta_laskuri(valitulos);
  nayta_laskuri(tulos);
}


/****************************************************************************/
void nollaa_laskurit(void)
{
  int i;
  for (i=0; i<LASKUREITA; i++) {
    laskurit[i] = 0;
    nayta_laskuri(i);
  }
  edel_oper = EI_OPER;
}


/****************************************************************************/
BOOL CALLBACK MainWndProc(HWND hWnd, unsigned message, WPARAM wParam, LPARAM lParam)
{
#pragma argsused
  GhWnd = hWnd;
  switch (message)
  {
    case WM_COMMAND: {
      WPARAM wp = GET_WM_COMMAND_ID(wParam,lParam); /* Jotta WIN32 toimisi */
      switch ( wp ) {
        case P1         : lisaa_laskuria(VALITULOSL);         return TRUE;
        case N0: case N1: case N2: case N3: case N4:
        case N5: case N6: case N7: case N8: case N9:
                          lisaa_numero(VALITULOSL,wp-N0);     return TRUE;
        case BS         : bs_numero(VALITULOSL);              return TRUE;
        case YNNAA      : oper(VALITULOSL,TULOSL,PLUS);       return TRUE;
        case VAHENNA    : oper(VALITULOSL,TULOSL,MIINUS);     return TRUE;
        case LASKETULOS : oper(VALITULOSL,TULOSL,LASKE);      return TRUE;
        case NOLLAA     : nollaa_laskurit();                  return TRUE;

        case IDCANCEL:
        case EXIT:
          PostQuitMessage(0);
          return TRUE;
      }
    }
  }
  return FALSE;
}


/****************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
#pragma argsused
  MSG     msg;
  FARPROC lProc  = MakeProcInstance((FARPROC)MainWndProc,hInstance);
  HWND    hWnd   = CreateDialog(hInstance,"LASKURI",NULL,(DLGPROC)lProc);
  HACCEL  hAccel = LoadAccelerators(hInstance,"PIKA");

  if ( !hWnd ) goto lopetus;
  while (GetMessage(&msg,NULL,NULL,NULL)) {
    if ( TranslateAccelerator(hWnd,hAccel,&msg) ) continue;
    if ( IsDialogMessage(hWnd,&msg)  )            continue;
    TranslateMessage(&msg);     /* Tulkitaan virtuaaliset np. koodit      */
    DispatchMessage(&msg);      /* Lhetetn viesti ikkunalle             */
  }
lopetus:
  (void)FreeProcInstance(lProc);
  return 0;
}