/**************/
/* kieli.c    */
/****************************************************************************
**  PROGRAM: kieli.c
**  PURPOSE: Erikielisten menujen kyttminen ja lipun piirminen
**           pikkunan alalaitaan.
**
**           Kyttjn tytyy tehd resurssitiedostoon
**           haluamansa kokoiset liput bittikarttoina nimille
**           KIELI_? , miss ? on juokseva numero 1:st eteenpin
**           Vastaavasti tehdn menut samoille nimille
**           ja mikli menuun halutaan lipunkuva, pit
**           kielet numeroida menun johonkin kohtaan
**           IDM_OPTIONS_LANGUAGE+? eteenpin, miss ? alkaa 0:sta.
**           Jos lipunkuva halutaan mys ikkunan alalaitaan,
**           pit se tehd User Draw push buttonina, jolle
**           annetaan tunnus ID_LANGUAGE.
**           Tunnukset lytyvt tiedostosta kieli.h
****************************************************************************/
#include <windows.h>
#include <stdio.h>
#include "..\menu4\kieli.h"


/***************************************************************************/
typedef struct {
  char    name[20];
  int     item;
  HBITMAP hBit;
  HMENU   hMenu;
} LanguageType;

/***************************************************************************/
#define MAX_KIELIA 10
static int KIELIA = 0;
static LanguageType Languages[MAX_KIELIA] = {
  {"",IDM_OPTIONS_LANGUAGE,NULL,NULL}
};
static int kieli=0;

/***************************************************************************/
void TaytaBittikarttalla(HWND hWnd,HDC hDC,HBITMAP hBitmap)
/* Tytetn koko alue bittikartalla                                       */
{
  HDC  hDCBitmap = CreateCompatibleDC(hDC);
  HGDIOBJ hOld  = SelectObject(hDCBitmap,hBitmap);
  RECT rc;
  BITMAP bm;

  GetObject(hBitmap,sizeof(bm),&bm);
  GetClientRect(hWnd,&rc);
  StretchBlt(hDC,0,0,rc.right,rc.bottom,hDCBitmap,
                 0,0,bm.bmWidth,bm.bmHeight,SRCCOPY);
  SelectObject(hDCBitmap,hOld);
  DeleteDC(hDCBitmap);
}


/***************************************************************************/
int AsetaKieli(HWND hWnd,int k)
{
  kieli = k;
  if ( kieli >= KIELIA ) kieli = 0;
  SetMenu(hWnd,Languages[kieli].hMenu);
  InvalidateRect(hWnd,NULL,FALSE);
  return TRUE;
}


/***************************************************************************/
BOOL language_message(HWND hWnd, unsigned message,
                      WPARAM wParam, LPARAM lParam)
{
  switch (message) {
    case WM_DRAWITEM:
      switch (wParam) {
        case ID_LANGUAGE: {
           DRAWITEMSTRUCT *dis = (DRAWITEMSTRUCT *)lParam;
           HWND hwnd = GetDlgItem(hWnd,ID_LANGUAGE);
           TaytaBittikarttalla(hwnd,dis->hDC,Languages[kieli].hBit);
           return TRUE;
        }
      }

      return FALSE;
      
    case WM_COMMAND:
      if ( ( IDM_OPTIONS_LANGUAGE <= wParam ) &&
           ( wParam < IDM_OPTIONS_LANGUAGE+KIELIA) )
        return AsetaKieli(hWnd,wParam-IDM_OPTIONS_LANGUAGE);

      if ( wParam == ID_LANGUAGE )
        return AsetaKieli(hWnd,++kieli);

  }

  return FALSE;

}

/***************************************************************************/
void AlustaMenut(HANDLE hInstance,HWND hWnd)
/* Ladataan tarvittavat bittikartat ja vaihdetaan ne kussakin menussa
** kunkin kielen valinnan tilalle.  Kielten valinnat tulee olla numeroitu
** juoksevasti.  Jos joku valinta puuttuu, listn se ensimmisen kielen
** ylpuolelle.  Jos jonkin kielisess menussa ei ole yhtn kielenvaihtoa,
** ei sinne niit listkn!
*/
{
  int i,j;

  for (i=0; i<MAX_KIELIA; i++) {
    sprintf(Languages[i].name,"kieli_%d",i);
    if ( (Languages[i].hBit    = LoadBitmap(hInstance,Languages[i].name))
          == NULL) break;
  }
  KIELIA = i;

  for (i=0; i<KIELIA; i++)
    Languages[i].hMenu    = LoadMenu(hInstance,Languages[i].name);

  for (i=0; i<KIELIA; i++) {
    /* Lippujen kuvat menuihin tekstien tilalle */
    int first_language = -1;
    for (j=0; j<KIELIA; j++)  /* Ensin muutetaan jos pystyy */
      if ( ModifyMenu(Languages[i].hMenu,IDM_OPTIONS_LANGUAGE+j,
                       MF_ENABLED | MF_BITMAP,IDM_OPTIONS_LANGUAGE+j,
                       (LPSTR)MAKELONG(Languages[j].hBit,0)) )
         if ( first_language == -1 ) first_language = j;

    if ( first_language >= 0 )  /* Listn puuttuvat kielet */
      for (j=0; j<KIELIA; j++)  /* Sitten listn puuttuvat */
        if ( j != i ) {
          if ( EnableMenuItem(Languages[i].hMenu,IDM_OPTIONS_LANGUAGE+j,
                              MF_BYCOMMAND | MF_ENABLED) == -1 )
            InsertMenu(Languages[i].hMenu,IDM_OPTIONS_LANGUAGE+first_language,
                       MF_ENABLED | MF_BITMAP,IDM_OPTIONS_LANGUAGE+j,
                       (LPSTR)MAKELONG(Languages[j].hBit,0));
        }
  }
  /* Varmistetaan, ett kieleksi tulee SUOMI                               */
  SendMessage(hWnd,WM_COMMAND,IDM_OPTIONS_LANGUAGE,0);
}

/***************************************************************************/
void PoistaMenut(void)
{
  int i;
  for (i=KIELIA-1; i>=0; i--) DestroyMenu(Languages[i].hMenu);
  for (i=KIELIA-1; i>=0; i--) DeleteObject(Languages[i].hBit);
}