/**************/
/* kieli.c    */
/****************************************************************************
**  PROGRAM: kieli.c
**  PURPOSE: Erikielisten menujen kyttminen ja lipun piirminen
**           pikkunan alalaitaan.
**
**           Projektiin lis
**             kieli.c
**             ALI/tausta.c  - bittikarttojen piirto
**             ALI/tabhand.c - jos kytetn viestitaulukon ksittely  
**
**           Kyttjn tytyy tehd resurssitiedostoon
**           haluamansa kokoiset liput bittikarttoina nimille
**           KIELI_? , miss ? on juokseva numero 1:st eteenpin
**           Vastaavasti tehdn menut samoille nimille
**           ja mikli menuun halutaan lipunkuva, pit
**           kielet numeroida menun johonkin kohtaan
**           IDM_OPTIONS_LANGUAGE+? eteenpin, miss ? alkaa 0:sta.
**           Jos lipunkuva halutaan mys ikkunan alalaitaan,
**           pit se tehd User Draw push buttonina, jolle
**           annetaan tunnus ID_LANGUAGE.
**           Tunnukset lytyvt tiedostosta kieli.h
**
**  Aliohjelmat: (nykyisin toimii vain yhdelle ikkunalle kerralla)
**    void AlustaKielet(HWND hWnd);
**      - alustaa kielet ikkunaa hWnd varten
**    void PoistaKielet(HWND hWnd);
**      - poistaa ikkunan hWnd kieli varten tehdyt varaukset
**        tt ei ole pakko kutsua, jos viestitaulukossa on rivi
**            LOOK_TABLE(LanguageMsgTbl),
**        Ko. rivi on muutenkin oltava viestitaulokossa, jotta
**        kielen vaihto ja lipun yllpito toimisi.
**    int AsetaKieli(HWND hWnd,int k);
**      - vaihdetaan kieleksi kieli numero k
**    int MikaKieli(HWND hWnd);
**      - kysytn mik kieli on tll hetkell kytss (esim ohjelman
**        lopussa talletusta varten).
**
** Suunnitelmia:
** -------------
**  - ikkunakohtaiset kielet
****************************************************************************/
#include <windows.h>
#include <stdio.h>
#include "tausta.h"
#include "tabhand.h"
#include "kieli.h"

/***************************************************************************/
typedef struct {
  char    name[20];
  int     item;
  HBITMAP hBit;
  HMENU   hMenu;
} LanguageType;

/***************************************************************************/
#define MAX_KIELIA 10
static int Kielia = 0;
static LanguageType Languages[MAX_KIELIA] = {
  {"",IDM_OPTIONS_LANGUAGE,NULL,NULL}
};
static int kieli=0;
static HWND hWnd_kieli = NULL; /* Mille ikkunalle kielet nyt kytss      */ 

/***************************************************************************/
int AsetaKieli(HWND hWnd,int k)
{
  kieli = k;
  if ( kieli >= Kielia ) kieli = 0;
  SetMenu(hWnd,Languages[kieli].hMenu);
  InvalidateRect(hWnd,NULL,FALSE);
  return 0;
}

/***************************************************************************/
int MikaKieli(HWND hWnd)
{
  if ( hWnd == hWnd_kieli ) return kieli;
  return 0;
}

/***************************************************************************/
void AlustaKielet(HWND hWnd)
/* Ladataan tarvittavat bittikartat ja vaihdetaan ne kussakin menussa
** kunkin kielen valinnan tilalle.  Kielten valinnat tulee olla numeroitu
** juoksevasti.  Jos joku valinta puuttuu, listn se ensimmisen kielen
** ylpuolelle.  Jos jonkin kielisess menussa ei ole yhtn kielenvaihtoa,
** ei sinne niit listkn!
*/
{
  int i,j;
  HINSTANCE hInstance = GetWindowInstance(hWnd);

  if ( hWnd_kieli ) return;
  hWnd_kieli = hWnd;

  for (i=0; i<MAX_KIELIA; i++) {
    sprintf(Languages[i].name,"kieli_%d",i);
    if ( (Languages[i].hBit    = LoadBitmap(hInstance,Languages[i].name))
          == NULL) break;
  }
  Kielia = i;

  for (i=0; i<Kielia; i++)
    Languages[i].hMenu    = LoadMenu(hInstance,Languages[i].name);

  for (i=0; i<Kielia; i++) {
    /* Lippujen kuvat menuihin tekstien tilalle */
    int first_language = -1;
    for (j=0; j<Kielia; j++)  /* Ensin muutetaan jos pystyy */
      if ( ModifyMenu(Languages[i].hMenu,IDM_OPTIONS_LANGUAGE+j,
                       MF_ENABLED | MF_BITMAP,IDM_OPTIONS_LANGUAGE+j,
                       (LPSTR)Languages[j].hBit) )
         if ( first_language == -1 ) first_language = j;

    if ( first_language >= 0 )  /* Listn puuttuvat kielet */
      for (j=0; j<Kielia; j++)  /* Sitten listn puuttuvat */
        if ( j != i ) {
          if ( EnableMenuItem(Languages[i].hMenu,IDM_OPTIONS_LANGUAGE+j,
                              MF_BYCOMMAND | MF_ENABLED) == -1 )
            InsertMenu(Languages[i].hMenu,IDM_OPTIONS_LANGUAGE+first_language,
                       MF_ENABLED | MF_BITMAP,IDM_OPTIONS_LANGUAGE+j,
                       (LPSTR)Languages[j].hBit);
        }
  }
  /* Varmistetaan, ett kieleksi tulee SUOMI                               */
  AsetaKieli(hWnd,0);
}

/***************************************************************************/
void PoistaKielet(HWND hWnd)
{
  int i;
  if ( hWnd_kieli != hWnd ) return;
  for (i=Kielia-1; i>=0; i--) DestroyMenu(Languages[i].hMenu);
  for (i=Kielia-1; i>=0; i--) DeleteObject(Languages[i].hBit);
  Kielia = 0;
}

/***************************************************************************/
static EVENT WM_drawitem_ID_LANGUAGE(tMSGParam *msg)
{
  DRAWITEMSTRUCT *dis = (DRAWITEMSTRUCT *)msg->lParam;
  HWND hwnd = GetDlgItem(msg->hWnd,ID_LANGUAGE);
  TaytaBittikartalla(hwnd,dis->hDC,Languages[kieli].hBit);
  return 0;
}

static EVENT WM_command_LANGUAGE(tMSGParam *msg) /* #man# */
{
  return AsetaKieli(msg->hWnd,msg->id - IDM_OPTIONS_LANGUAGE);
}

static EVENT WM_command_ID_LANGUAGE(tMSGParam *msg)
{
  return AsetaKieli(msg->hWnd,++kieli);
}

static EVENT WM_destroy(tMSGParam *msg)
{
  PoistaKielet(msg->hWnd);
  return CONTINUE_TABLE;
}

tMSGEntry LanguageMsgTbl[] = {
  { WM_COMMAND, TRANGE, DoC, WM_command_LANGUAGE,0,0, IDM_OPTIONS_LANGUAGE,IDM_OPTIONS_LANGUAGE+MAX_KIELIA-1},
  { WM_DRAWITEM , ID_LANGUAGE , DoC , WM_drawitem_ID_LANGUAGE }, /*a*/
  { WM_COMMAND , ID_LANGUAGE , DoC , WM_command_ID_LANGUAGE }, /*a*/
  { WM_DESTROY , DoC , DoC , WM_destroy }, /*a*/
  { 0 }
};
