/**************/
/* tiedleik.c */
/****************************************************************************
**  PROGRAM: tiedleik.c
**  PURPOSE: Autolaskurin tiedoston ksittely ja leikekirjan ksittely
****************************************************************************/
#include <windows.h>            /* Tarvitaan kaikissa Windows C-ohjelmissa */
#include <stdio.h>
#include <io.h>
#include <string.h>
#include "filename.h"
#include "clipboard.h"
#include "laskurm4.h"

/***************************************************************************/
int laskurit[LASKUREITA] = {0,0};

/***************************************************************************/
int ShowStatic(HWND hWnd, int id, int value)
/* Aliohjelmalla tulostetaan Static ikkunaan numero                        */
{
  char s[20];
  wsprintf(s,"%d ",value);
  SetDlgItemText(hWnd,id,s);
  return 0;
}


/***************************************************************************/
int nollaa_laskurit(HWND hWnd,int laskurit[])
{
  int i;
  for (i=0; i<LASKUREITA; i++) ShowStatic(hWnd,HAL+i,laskurit[i] = 0);
  return 0;
}

/***************************************************************************/
/* Datan muuttaminen tekstiksi ja pinvastoin:                             */
static char laskuri_teksti[128];

/*-------------------------------------------------------------------------*/
char *laskurit_tekstiksi(int *laskurit)
/* Muutetaan laskureiden arvo tekstiksi                                    */
{
  sprintf(laskuri_teksti,"%d %d",laskurit[0],laskurit[1]);
  return laskuri_teksti;
}

/*-------------------------------------------------------------------------*/
int teksti_laskureiksi(HWND hWnd,const char *teksti,int *laskurit)
/* Muutetaan teksti laskureiksi                                            */
{
  int i;
  sscanf(teksti,"%d %d",&laskurit[0],&laskurit[1]);
  for (i=0; i<LASKUREITA; i++) ShowStatic(hWnd,HAL+i,laskurit[i]);
  return 0;
}


/***************************************************************************/
/* Tiedoston ksittely varten tarvittavat funktiot ja tietue:             */
/*-------------------------------------------------------------------------*/
static int MySave(FileType *File)
{
  static OFSTRUCT OfStruct;
  int count,n;
  static char *s;
  HFILE hFile = OpenFile(File->Name,&OfStruct,OF_CREATE);

  if ( hFile == HFILE_ERROR )  goto error;

  s     = laskurit_tekstiksi((int *)File->data);
  count = strlen(s);
  n     = _lwrite(hFile,s,count);

  _lclose(hFile);
  if ( n == count ) return 0;

error:
  MessageBox(File->hWnd,File->Name,"Talletus eponnistui",
             MB_APPLMODAL |  MB_OK | MB_ICONEXCLAMATION);
  return -1;
}

/*-------------------------------------------------------------------------*/
static int MyRead(FileType *File)
{
  static OFSTRUCT OfStruct;
  HFILE hFile = OpenFile(File->Name,&OfStruct,OF_READ);
  int n;
  static char s[128];

  if ( hFile == HFILE_ERROR )  goto error;

  n = _lread(hFile,s,sizeof(s));
  _lclose(hFile);
  teksti_laskureiksi(File->hWnd,s,(int *)File->data);
  if ( n > 0 ) return 0;

error:
  MessageBox(File->hWnd,File->Name,"Lukeminen eponnistui",
             MB_APPLMODAL |  MB_OK | MB_ICONEXCLAMATION);
  return -1;
}

/*-------------------------------------------------------------------------*/
static int MyNew(FileType *File)
{
  nollaa_laskurit(File->hWnd,(int *)File->data);
  return 0;
}

/*-------------------------------------------------------------------------*/
FileType File={
  "",                           /* Tiedoston nimi, aluksi ""               */
  WTITLE,                       /* Ikkunan otsikko, jonka pern tied.nimi */
  1,                            /* Luetun funktion tyyppi, ks. szFilter    */
  "Laskuri (*.las)\0*.las\0Teksti (*.txt)\0*.txt\0Kaikki (*.*)\0*.*\0",
  0,                            /* Onko tiedostoa muutettu.                */
  MySave,                       /* Osoitin talletusfunktioon               */
  MyRead,                       /* Osoitin lukufunktioon                   */
  MyNew,                        /* Osoitin tiedon nollausfunktioon         */
  laskurit                      /* osoitin dataan                          */
};

/***************************************************************************/
int laskurit_leikekirjaan(HWND hWnd,int *laskurit)
{
  char *pText = laskurit_tekstiksi(laskurit);
  return TextToClipboard(hWnd,pText);
}

/***************************************************************************/
int leikekirja_laskureihin(HWND hWnd,int *laskurit)
{
  const char *pText = TextClipboard(hWnd);
  if ( !pText ) return -1;
  teksti_laskureiksi(hWnd,pText,laskurit);
  return 0;
}